shared float4x4 World;
shared float4x4 View;
shared float4x4 Projection;

shared float3 cameraPosition;

//light properties
shared float3 lightPosition;
shared float4 ambientLightColor;
shared float4 diffuseLightColor;
shared float4 specularLightColor;

//material properties
shared float specularPower;
shared float specularIntensity;

texture ModelTexture;

sampler TextureSampler = 
sampler_state
{
    Texture = <ModelTexture>;
    MipFilter = LINEAR;
    MinFilter = LINEAR;
    MagFilter = LINEAR;
};

texture NormalTexture;

sampler NormalSampler = 
sampler_state
{
    Texture = <NormalTexture>;
    MipFilter = LINEAR;
    MinFilter = LINEAR;
    MagFilter = LINEAR;
}; 

struct VertexShaderInput
{
    float4 Position : POSITION0;
	float2 TexCoord : TEXCOORD0;
};

struct VertexShaderOutput
{
    float4 Position : POSITION0;
    float3 WorldPosition : TEXCOORD1;
	float2 TexCoord : TEXCOORD2;
};

VertexShaderOutput VertexShaderFunction(VertexShaderInput input)
{
    VertexShaderOutput output;

    float4 worldPosition = mul(input.Position, World);
	output.WorldPosition = worldPosition;
    float4 viewPosition = mul(worldPosition, View);
    output.Position = mul(viewPosition, Projection);

	output.TexCoord = input.TexCoord;

    return output;
}

float4 PixelShaderFunction(VertexShaderOutput input) : COLOR0
{
	float3 normal = 2 * (tex2D(NormalSampler, input.TexCoord) - 0.5);
	normal.xyz = float3(-normal.x, normal.z, -normal.y);
	float3 directionToLight = normalize(lightPosition - input.WorldPosition);
	float diffuseIntensity = saturate( dot(directionToLight, normal));
	float4 diffuse = diffuseLightColor * diffuseIntensity;

	//calculate Phong components per-pixel
	float3 reflectionVector = normalize(reflect(-directionToLight, normal));
	float3 directionToCamera = normalize(cameraPosition - input.WorldPosition);

	//calculate specular component
	float4 specular = specularLightColor * specularIntensity * 
				   pow(saturate(dot(reflectionVector, directionToCamera)), 
					   specularPower);

	//all color components are summed in the pixel shader
	float4 Color = (specular  + diffuse + ambientLightColor) * tex2D(TextureSampler, input.TexCoord);
	Color.a = 1.0;
	
	return Color;
}

technique Technique1
{
    pass Pass1
    {
        // TODO: set renderstates here.

        VertexShader = compile vs_1_1 VertexShaderFunction();
        PixelShader = compile ps_2_0 PixelShaderFunction();
    }
}
